@@ -29,7 +29,8 @@ class specTrails(object):
 #        #
         # ---------------------------------------------------------------------------
         #
-    def ionGate(self, elementList = None, ionList = None, minAbund=None, doLines=1, doContinuum=1, doWvlTest=1, doIoneqTest=1, includeDiel=False,  verbose=0):
+    def ionGate(self, elementList = None, ionList = None, minAbund=None, doLines=1, doContinuum=1,
+        doWvlTest=1, doIoneqTest=1, includeDiel=False,  verbose=0):
         '''
         creates a list of ions for free-free, free-bound, and line intensity calculations
         if doing the radiative losses, accept all wavelength -> doWvlTest=0
@@ -46,12 +47,13 @@ class specTrails(object):
                 minAbund = float(minAbundAll)
         ionInfo = chio.masterListInfo()
         #
-        if hasattr(self, 'Wavelength'):
-            wvlRange = [self.Wavelength.min(), self.Wavelength.max()]
-        elif hasattr(self, 'WvlRange'):
-            wvlRange = self.WvlRange
-        else:
-            print(' need a wavelength range in ionGate ')
+        if doWvlTest:
+            if hasattr(self, 'Wavelength'):
+                wvlRange = [self.Wavelength.min(), self.Wavelength.max()]
+            elif hasattr(self, 'WvlRange'):
+                wvlRange = self.WvlRange
+            else:
+                print(' need a wavelength range in ionGate ')
         #
         temperature = self.Temperature
         #
@@ -154,15 +156,34 @@ class specTrails(object):
             toPop = []
             for ionS in todo:
                 # bare ions have wmin=0 and wmax=1.e+30
+
+                if self.Defaults['wavelength'] == 'angstrom':
+                    wmin = ionInfo[ionS]['wmin']
+                    wmax = ionInfo[ionS]['wmax']
+                elif self.Defaults['wavelength'] == 'nm':
+                    wmin = ionInfo[ionS]['wmin']/10.
+                    wmax = ionInfo[ionS]['wmax']/10.
+                elif self.Defaults['wavelength'] == 'ev':
+                    wmin = const.ev2Ang/ionInfo[ionS]['wmax']
+                    wmax = const.ev2Ang/ionInfo[ionS]['wmin']
+                elif self.Defaults['wavelength'] == 'kev':
+                    wmin = const.kev2Ang/ionInfo[ionS]['wmax']
+                    wmax = const.kev2Ang/ionInfo[ionS]['wmin']
+
+                if verbose:
+                    print('%s wmin:  %10.2e  wmax:  %10.2e'%(ionS,  wmin,  wmax))
+
                 if doWvlTest:
-                    wvlTestMin = wvlRange[0] <= ionInfo[ionS]['wmax']
-                    wvlTestMax = wvlRange[1] >= ionInfo[ionS]['wmin']
+                    wvlTestMin = wvlRange[0] <= wmax
+#                    print(' wvlRange[0] %10.2e <= wmax %10.2e'%(wvlRange[0],  wmax))
+                    wvlTestMax = wvlRange[1] >= wmin
+#                    print(' wvlRange[1] %10.2e >= wmin %10.2e'%(wvlRange[1],  wmin))
                 else:
                     wvlTestMin = 1
                     wvlTestMax = 1
-#                if verbose:
-#                    print(' %s  %8.2f  %8.2f %8.2f %8.2f'%(ionS, ionInfo[ionS]['wmin'], ionInfo[ionS]['wmax'], wvlRange[0], wvlRange[1]))
-#                    print(' %s wvlTestMin  %s  wvlTestMax %s'%(ionS, wvlTestMin, wvlTestMax))
+                if verbose:
+                    print(' %s  %8.2f  %8.2f %8.2f %8.2f'%(ionS, ionInfo[ionS]['wmin'], ionInfo[ionS]['wmax'], wvlRange[0], wvlRange[1]))
+                    print(' %s wvlTestMin  %s  wvlTestMax %s'%(ionS, wvlTestMin, wvlTestMax))
                 if wvlTestMin and wvlTestMax:
                     if verbose:
                         print(' %s passed wvl test'%(ionS))
@@ -370,6 +391,7 @@ class specTrails(object):
 
         top:  integer
             specifies to plot only the top strongest lines, default = 10
+            if set to None, all lines are plotted
 
         '''
         fs = 14
@@ -422,7 +444,7 @@ class specTrails(object):
             index = 0
             lineIntensity = self.Intensity['intensity'][0]
             if hasattr(self, 'Continuum'):
-                continuum = self.Continuum['intensity'][0]
+                continuum = self.Continuum['intensity']
             else:
                 continuum = np.zeros_like(lineIntensity)
             lineWvl = self.Intensity['wvl']
@@ -454,8 +476,10 @@ class specTrails(object):
             self.Error = 1
             self.Message = 'No lines in this wavelength interval'
             return
-        elif top == 0:
-            top = lineWvl.size
+        elif top is None:
+#            top = lineWvl.size
+            top = 0
+            lineWvl = []
         elif lineWvl.size > top:
             intsrt = np.argsort(lineIntensity)
             lineWvl = lineWvl[intsrt[-top:]]
