@@ -7,6 +7,7 @@ import os
 
 import numpy as np
 from scipy.interpolate import splev, splrep
+from scipy import special
 from scipy.ndimage import map_coordinates
 #import matplotlib.pyplot as plt
 #import ChiantiPy.core as ch
@@ -73,9 +74,14 @@ class continuum(ionTrails):
     """
 
     def __init__(self, ionStr,  temperature, abundance=None, em=None, verbose=0):
+
+
         self.IonStr = ionStr
         self.nameDict = util.convertName(ionStr)
+        #  Z is the ion charge
         self.Z = self.nameDict['Z']
+        # Zion is the charge of the ion
+        self.Zion = self.nameDict['Ion'] -1
         self.Stage = self.nameDict['Ion']
         self.Ion = self.nameDict['Ion']
         self.Dielectronic = self.nameDict['Dielectronic']
@@ -83,9 +89,15 @@ class continuum(ionTrails):
         self.NTemperature = self.Temperature.size
         self.Defaults = chdata.Defaults
 
+        if self.Defaults['wavelength'] != 'angstrom':
+            print(' the continuum can only be calculated for wavelengths in angstroms')
+            return
 
         self.argCheck(temperature=temperature, eDensity=None, pDensity=None, em=em, verbose=verbose)
 
+        self.Labels = util.units(chdata.Defaults)
+
+
         self.Ip = chdata.Ip[self.Z-1, self.Stage-1]
         self.Ipr = chdata.Ip[self.Z-1, self.Stage-2]
         self.ionization_potential = chdata.Ip[self.Z-1, self.Stage-1]*const.ev2Erg
@@ -142,7 +154,7 @@ class continuum(ionTrails):
         if includeIoneq:
             prefactor *= self.ioneq_one(self.Stage, **kwargs)
 
-        self.FreeFreeLoss = prefactor*(self.Z**2)*np.sqrt(self.Temperature)*gaunt_factor
+        self.FreeFreeLoss = prefactor*(self.Zion**2)*np.sqrt(self.Temperature)*gaunt_factor
 
     def freeFree(self, wavelength, includeAbund=True, includeIoneq=True, **kwargs):
         """
@@ -177,15 +189,16 @@ class continuum(ionTrails):
             If True, include the ionization equilibrium in the final output
 
         """
-        if self.Em.max() == 1.:
-            ylabel = 'erg cm$^{-2}$ s$^{-1}$ sr$^{-1}$ \u212B$^{-1}$ ($\int\,$ N$_e\,$N$_H\,$d${\it l}$)$^{-1}$'
-        else:
-            ylabel = 'erg cm$^{-2}$ s$^{-1}$ sr$^{-1}$ \u212B$^{-1}$'
+        if self.Defaults['wavelength'] != 'angstrom':
+            print(' the continuum can only be calculated for wavelengths in angstroms')
+            return
 
-        if self.Defaults['wavelength'] == 'angstrom':
-            xlabel = 'Wavelength \u212B'
-        else:
-            xlabel = r'Wavelength ('+self.Defaults['wavelength'] +')'
+
+        xlabel = self.Labels['xlabel']
+        ylabel = self.Labels['spectrumYlabel']
+
+        if np.array_equal(self.Em, np.ones_like(self.Em)):
+            ylabel += '($\int\,$ N$_e\,$N$_H\,$d${\it l}$)$^{-1}$'
 
         wavelength = np.atleast_1d(wavelength)
         # define the numerical prefactor
@@ -193,7 +206,7 @@ class continuum(ionTrails):
                      * (const.fine*const.planck/np.pi)**3
                      * np.sqrt(2.*np.pi/3./const.emass/const.boltzmann))
         # include temperature dependence
-        prefactor *= self.Z**2/np.sqrt(self.Temperature)
+        prefactor *= self.Zion**2/np.sqrt(self.Temperature)
         if includeAbund:
             prefactor *= self.Abundance
         if includeIoneq:
@@ -214,7 +227,7 @@ class continuum(ionTrails):
 
         free_free_emission = (prefactor[:,np.newaxis]*exp_factor*gf/energy_factor).squeeze()
         self.FreeFree = {'intensity':free_free_emission, 'temperature':self.Temperature, 'wvl':wavelength,
-            'em':self.Em, 'ions':self.IonStr, 'xlabel':xlabel,  'ylabel':ylabel}
+            'em':self.Em, 'ions':self.IonStr, 'xlabel':xlabel,  'ylabel':ylabel,  'gf':gf}
 
     def freeFreeLoss(self, includeAbund=True, includeIoneq=True,  **kwargs):
         """
@@ -227,21 +240,21 @@ class continuum(ionTrails):
         .. math::
            \\frac{dW}{dtdV} = \\frac{4\\alpha^3h^2}{3\pi^2m_e}\left(\\frac{2\pi k_B}{3m_e}\\right)^{1/2}Z^2T^{1/2}\\bar{g}_B
 
-        where where :math:`Z` is the nuclear charge, :math:`T` is the electron temperature, and
+        where where :math:`Z` is the ion charge, :math:`T` is the electron temperature, and
         :math:`\\bar{g}_{B}` is the wavelength-averaged and velocity-averaged Gaunt factor. The
         Gaunt factor is calculated using the methods of [103]_. Note that this expression for the
         loss rate is just the integral over wavelength of Eq. 5.14a of [103]_, the free-free emission, and
         is expressed in units of erg :math:`\mathrm{cm}^3\,\mathrm{s}^{-1}`.
 
         """
-        # interpolate wavelength-averaged K&L gaunt factors
+        # interpolate wavelength-averaged  gaunt factors of Sutherland
         gf_kl_info = io.gffintRead()
         gamma_squared = self.IprErg/const.boltzmann/self.Temperature
 #        for i, atemp in enumerate(self.Temperature):
 #            print('%s T:  %10.2e gamma_squared  %10.2e'%(self.IonStr, atemp, gamma_squared[i]))
         gaunt_factor = splev(np.log(gamma_squared),
                              splrep(gf_kl_info['g2'],gf_kl_info['gffint']), ext=3)
-        # calculate numerical constant
+        # calculate numerical constant = Sutherland's Fk
         prefactor = (4.*(const.fine**3)*(const.planck**2)/3./(np.pi**2)/const.emass
                      * np.sqrt(2.*np.pi*const.boltzmann/3./const.emass))
         # include abundance and ionization equilibrium
@@ -250,8 +263,9 @@ class continuum(ionTrails):
         if includeIoneq:
             prefactor *= self.IoneqOne
 
-
-        self.FreeFreeLoss = {'rate':prefactor*(self.Z**2)*np.sqrt(self.Temperature)*gaunt_factor}
+        rate = prefactor*(self.Zion**2)*np.sqrt(self.Temperature)*gaunt_factor
+        self.FreeFreeLoss = {'temperature':self.Temperature, 'rate':rate,  'gf':gaunt_factor,
+            'prefactor':prefactor}
 
 
     def itoh_gaunt_factor(self, wavelength):
@@ -283,12 +297,17 @@ class continuum(ionTrails):
         upper_u = 1./2.5*(np.log10(lower_u) + 1.5)
         t = 1./1.25*(np.log10(self.Temperature) - 7.25)
         # read in Itoh coefficients
-        itoh_coefficients = io.itohRead()['itohCoef'][self.Z - 1].reshape(11,11)
+        itoh_coefficients = io.itohRead()['itohCoef'][self.Zion - 1].reshape(11,11)
         # calculate Gaunt factor
         gf = np.zeros(upper_u.shape)
-        for j in range(11):
-            for i in range(11):
-                gf += (itoh_coefficients[i,j]*(t**i))[:,np.newaxis]*(upper_u**j)
+        if self.Ntemp == 1:
+            for j in range(11):
+                for i in range(11):
+                    gf += (itoh_coefficients[i,j]*(t**i))*(upper_u**j)
+        else:
+            for j in range(11):
+                for i in range(11):
+                    gf += (itoh_coefficients[i,j]*(t**i))[:,np.newaxis]*(upper_u**j)
         # apply NaNs where Itoh approximation is not valid
         gf = np.where(np.logical_and(np.log10(lower_u) >= -4., np.log10(lower_u) <= 1.0),gf,np.nan)
         gf[np.where(np.logical_or(np.log10(self.Temperature) <= 6.0,
@@ -308,7 +327,7 @@ class continuum(ionTrails):
         """
         # calculate scaled quantities
         lower_u = const.planck*(1.e8*const.light)/const.boltzmann/np.outer(self.Temperature,wavelength)
-        gamma_squared = (self.Z**2)*const.ryd2erg/const.boltzmann/self.Temperature[:,np.newaxis]*np.ones(lower_u.shape)
+        gamma_squared = (self.Zion**2)*const.ryd2erg/const.boltzmann/self.Temperature[:,np.newaxis]*np.ones(lower_u.shape)
         # convert to index coordinates
         i_lower_u = (np.log10(lower_u) + 4.)*10.
         i_gamma_squared = (np.log10(gamma_squared) + 4.)*5.
@@ -316,7 +335,7 @@ class continuum(ionTrails):
         gf_sutherland_data = io.gffRead()
         # interpolate data to scaled quantities
         gf_sutherland = map_coordinates(gf_sutherland_data['gff'],
-                                        [i_gamma_squared.flatten(), i_lower_u.flatten()]).reshape(lower_u.shape)
+            [i_gamma_squared.flatten(), i_lower_u.flatten()]).reshape(lower_u.shape)
 
         return np.where(gf_sutherland < 0., 0., gf_sutherland)
 
@@ -534,8 +553,7 @@ class continuum(ionTrails):
             #
 
 
-
-    def freeBoundLoss(self,  includeAbund=True, includeIoneq=True,  verbose=False):
+    def freeBoundLoss(self,  includeAbund=True, includeIoneq=True, verner=True,  verbose=False):
         '''
         to calculate the free-bound (radiative recombination) energy loss rate coefficient of an ion,
         the ion is taken to be the target ion,
@@ -558,6 +576,13 @@ class continuum(ionTrails):
         .. [2] Verner & Yakovlev, 1995, A&AS, `109, 125
             <http://adsabs.harvard.edu/abs/1995A%26AS..109..125V>`_
          '''
+        if self.Defaults['wavelength'] != 'angstrom':
+            print(' the continuum can only be calculated for wavelengths in angstroms')
+            return
+
+        xlabel =  self.Labels['radlossTlabel']
+        ylabel = self.Labels['radlossYlabel']
+
         temperature = self.Temperature
 #        tev = const.boltzmannEv*temperature
         em = self.Em
@@ -644,24 +669,36 @@ class continuum(ionTrails):
 #        K_4 = \frac{1}{\pi} (\frac{1}{2 \pi m_e k})^{1/2}
         K4 = (1./np.pi)*(1./(np.sqrt(2.*np.pi*const.emass*const.boltzmann)))
 
-        fbLoss = np.zeros((nTemp,  nlvls),np.float64)
+        fbLoss = np.zeros((nTemp),np.float64)
         ratg = np.zeros((nlvls),np.float64)
         iprLvlErg = np.zeros((nlvls),np.float64)
         gfGl = np.zeros((nTemp, nlvls, const.ngl), np.float64)
         gfInt = np.zeros((nTemp,  nlvls), np.float64)
-        gfValues = np.zeros((nTemp, nlvls, const.ngl), np.float64)
-#        gfIntSum = np.zeros((nTemp), np.float64)
         gfIntAllSum = np.zeros((nTemp), np.float64)
         egl = np.zeros((nTemp, nlvls, const.ngl), np.float64)
         scaledE = np.zeros((nTemp, nlvls, const.ngl), np.float64)
 
         peAll = []
         gfAll = []
-        ratg[0] = float(multr[0])/float(mult[0])
-        iprLvlEv = self.Ipr - const.invCm2Ev*ecm[0]
-#        iprLvlErg = const.ev2Erg*iprLvlEv
-        # without verner
-        lvl1 = 0
+        V1 = (1./(const.emass*const.light**2))*np.sqrt(2./(np.pi*const.emass*const.boltzmann))
+        if verner:
+            ilvl = 0
+            lvl1 = 1
+            ratg[0] = float(multr[0])/float(mult[0])
+            iprLvlEv = self.Ipr - const.invCm2Ev*ecm[0]
+            iprLvlErg[ilvl] = const.ev2Erg*iprLvlEv
+            xgl = const.xgl
+            wgl = const.wgl
+
+            for itemp, atemp in enumerate(temperature):
+                hnu = xgl*const.boltzmann*atemp + iprLvlErg[ilvl]
+                wvl = 1.e+8*const.planck*const.light/hnu
+                self.vernerCross(wvl)
+                fbInt = wgl*(xgl*const.boltzmann*atemp + iprLvlErg[ilvl])**3*self.VernerCross
+                fbLoss[itemp] = em[itemp]*abund*ioneq[itemp]*V1*ratg[0]*fbInt.sum()/np.sqrt(atemp)
+        else:
+            # without verner
+            lvl1 = 0
         for ilvl in range(lvl1,nlvls):
             pqnIdx = pqn[ilvl] - 1
             lIdx = l[ilvl]
@@ -671,7 +708,6 @@ class continuum(ionTrails):
             gf = klgbf['klgbf'][lIdx]
             gfAll.append(gf)
             iprLvlEv = self.Ipr - const.invCm2Ev*ecm[ilvl]
-#            edgeLvlAng.append(const.ev2Ang/iprLvlEv)
             iprLvlErg[ilvl] = const.ev2Erg*iprLvlEv
             ratg[ilvl] = float(multr[ilvl])/float(mult[0]) # ratio of statistical weights
 
@@ -688,15 +724,270 @@ class continuum(ionTrails):
         for itemp,  atemp in enumerate(temperature):
             for ilvl in range(lvl1,nlvls):
                 gfIntAllSum[itemp] += ratg[ilvl]*iprLvlErg[ilvl]**2*gfIntSum[itemp]/float(pqn[ilvl])
+        fbkl = em*ioneq*abund*K1*K2*K4*gfIntAllSum/np.sqrt(temperature)
+        fbLoss += fbkl
 
-        fbloss = em*ioneq*abund*K1*K2*K4*gfIntAllSum/np.sqrt(temperature)
-        self.FreeBoundLoss = {'rate':fbloss, 'temperature':temperature}
+        self.FreeBoundLoss = {'rate':fbLoss, 'fbkl':fbkl, 'temperature':temperature,
+            'xlabel':xlabel, 'ylabel':ylabel}
 
-        #{'rate':fbloss, 'gfIntSum':gfIntSum,'gfInt':gfInt,'gfGL':gfGl, 'egl':egl, 'scaledE':scaledE,'peAll':peAll, 'gfAll':gfAll, 'gfIntAllSum':gfIntAllSum}
+
+    def freeBoundRate(self, verner=True, verbose=False):
+        """
+        Calculates the free-bound (radiative recombination) rate of an ion.
+        Provides emissivity in units of ergs :math:`\mathrm{cm}^{-2}` :math:`\mathrm{s}^{-1}` :math:`\mathrm{str}^{-1}` :math:`\mathrm{\AA}^{-1}` for an individual ion.  If includeAbund is set,
+        the abundance is included.  If includeIoneq is set, the ionization equililibrium for the given
+        ion is included
+        This method is only for the purposes of testing.
+        The best rr/FB rates come from the ion class
+
+        Notes
+        -----
+        - Uses the Gaunt factors of [103]_ for recombination to the excited levels
+        - Uses the photoionization cross sections of [3]_ to develop the free-bound cross section
+        - Include the elemental abundance and ionization fraction by default
+        - The specified ion is the target ion
+        - uses the corrected version of the K-L bf gaunt factors available in CHIANTI V10
+        - revised to calculate the bf cross, fb cross section and the maxwell energy distribution
+
+        """
+
+        if self.Defaults['wavelength'] != 'angstrom':
+            print(' the continuum can only be calculated for wavelengths in angstroms')
+            return
+
+        temperature = self.Temperature
+
+        ngl = 15
+        zgl = special.roots_laguerre(ngl)
+        xgl = zgl[0]
+        wgl = zgl[1]
+
+        xlabel = self.Labels['rateTlabel']
+        ylabel = self.Labels['rateYlabel']
+
+#        if np.array_equal(self.Em, np.ones_like(self.Em)):
+#            ylabel += '($\int\,$ N$_e\,$N$_H\,$d${\it l}$)$^{-1}$'
+
+
+        #
+        # the target ion contains that data for fblvl
+        #
+        if hasattr(self,'Fblvl'):
+            fblvl = self.Fblvl
+            if 'errorMessage' in fblvl.keys():
+                self.FreeBound = fblvl
+                return
+        elif self.Z == self.Stage-1:
+            #dealing with the fully ionized stage
+            self.Fblvl = {'mult':[1., 1.]}
+            fblvl = self.Fblvl
+        else:
+            fblvlname = self.nameDict['filename']+'.fblvl'
+            if os.path.isfile(fblvlname):
+                self.Fblvl = io.fblvlRead(self.IonStr)
+                fblvl = self.Fblvl
+            # in case there is no fblvl file
+            else:
+                if verbose:
+                    print(' No fblvl file for ion %s'%(self.IonStr))
+                self.FreeBound = {'errorMessage':' No fblvl file for ion %s'%(self.IonStr)}
+                return
+        #
+        #  need data for the recombined ion
+        #
+        if hasattr(self,'rFblvl'):
+            rFblvl = self.rFblvl
+        else:
+            lower = self.nameDict['lower']
+            lowerDict = util.convertName(lower)
+            rFblvlname = lowerDict['filename'] +'.fblvl'
+            if os.path.isfile(rFblvlname):
+                self.rFblvl = io.fblvlRead(lower)
+                rFblvl = self.rFblvl
+            else:
+                if verbose:
+                    print(' No fblvl file for ion %s'%(lower))
+                self.FreeBound = {'errorMessage':' no fblvl file for ion %s'%(lower)}
+                return
+        #
+        # for the ionization potential, must use that of the recombined ion
+        #
+        nlvls = len(rFblvl['lvl'])
+        # pqn = principle quantum no. n
+        pqn = np.asarray(rFblvl['pqn'], 'int64')
+        conf = rFblvl['conf']
+
+        # l is angular moment quantum no. L
+        l = rFblvl['l']
+        # energy level in inverse cm
+        ecm = rFblvl['ecm']
+        # statistical weigths/multiplicities
+        multr = rFblvl['mult']
+        mult = fblvl['mult']
+
+        # get revised karzas-latter Gaunt factors
+        klgbfn = chdata.Klgbfn
+        #
+        nWvl = ngl
+        nTemp = temperature.size
+        #
+        #  the verner cross-section is included in this version
+#        if verner:
+#            self.vernerCross(wvl)
+#            vCross = self.VernerCross
+#            lvl1 = 1
+#        else:
+#            lvl1 = 0
+#        lvl1 = 0
+#        expfun = np.zeros((nlvls,nTemp,nWvl),np.float64)
+#        mask = np.zeros((nlvls,nTemp,nWvl),np.bool_)
+        fbn = np.zeros((nlvls,nTemp,nWvl),np.float64)
+        fbIntensity = np.zeros((nlvls,nTemp,nWvl),np.float64)
+        ratg = np.zeros((nlvls),np.float64)
+#        mygf = np.zeros((nlvls), np.float64)
+#        ratg[0] = float(multr[0])/float(mult[0])
+        iprLvlEv = self.Ipr - const.invCm2Ev*ecm[0]
+        iprLvlErg = const.ev2Erg*iprLvlEv
+        edgeLvlAng = []
+        mygfInt = np.zeros(nlvls,  np.float64)
+        fbRate = np.zeros((nlvls,  nTemp), np.float64)
+#        hnu = const.planck*const.light/(1.e-8*wvl)
+#        hnuEv = const.ev2Ang/wvl
+        for ilvl in range(nlvls):
+            ratg[ilvl] = float(multr[ilvl])/float(mult[0]) # ratio of statistical weights
+
+        # constants for free bound
+        # K_1 = 1.726 \times 10^{-28} = \frac{2^4 h e^2 }{3 \sqrt 3 m_e c}
+        K1 = 2.**4*const.planck*const.q**2/(3.*np.sqrt(3.)*const.emass*const.light)
+        # K_2 = 6.107 \times 10^5 = \frac{1}{ 2 m_e c^2 }
+        K2 = 1./(2.*const.emass*const.light**2)
+
+        #  \frac{1}{h c} (\frac{1}{2 m_e})^{1/2} (\frac{1}{\pi k})^{3/2}
+#        K3 = (1./(const.planck*const.light))*(1./(np.sqrt(2.*const.emass)))*(
+#            1./(np.pi*const.boltzmann)**1.5)
+        K5 = np.sqrt(8./(np.pi*const.emass*const.boltzmann))
+        K0 = K1*K2*K5
+
+        # restrict calculations to temperatures where the ionization equ. is non-zero
+#        goodT = [ it for it, anioneq in enumerate(self.IoneqOne) if anioneq > 0.]
+        if verner:
+#            vrrRate = np.zeros((nTemp), np.float64)
+            V3 = (4.*np.pi)/(const.light**2)*(1./(2.*np.pi*const.emass))**(1.5)*np.sqrt(1./const.boltzmann)
+            ilvl = 0
+            lvl1 = 1
+            iprLvlEv = self.Ipr - const.invCm2Ev*ecm[ilvl]
+            edgeLvlAng.append(const.ev2Ang/iprLvlEv)
+
+            for it, atemp in enumerate(temperature):
+                xkt = xgl*const.boltzmann*atemp
+                hnu = xkt + iprLvlErg
+                wvl = (1.e8*const.planck*const.light)/hnu
+                self.vernerCross(wvl)
+                vernerSum = (wgl*(hnu)**2*self.VernerCross).sum()
+                fbRate[0, it] = V3*ratg[0]*vernerSum/np.sqrt(atemp)
+#                print(' verner %5i %5i  %10.2e'%(ilvl, itemp, atemp))
+#                mask[0,itemp] = 1.e+8/wvl < (self.IprCm - ecm[0])
+#                expfun[0, itemp] = np.exp((iprLvlErg - 1.e+8*const.planck*const.light/wvl)/
+#                    (const.boltzmann*atemp))
+#                fbn[0, itemp] = (const.planck*const.light/(1.e-8*wvl))**5 \
+#                    *const.verner*ratg[0]*expfun[0,itemp]*vCross/atemp**1.5
+#                fbIntensity[0,itemp] = abund*ioneq[itemp]*em[itemp]*fbn[0,  itemp]
+
+        else:
+            lvl1 = 0
+
+        for ilvl in range(lvl1,nlvls):
+            pqnIdx = pqn[ilvl] - 1
+            lIdx = l[ilvl]
+            klgbf = klgbfn[pqnIdx]
+            pe = klgbf['pe']
+            gf = klgbf['klgbf'][lIdx]
+            iprLvlEv = self.Ipr - const.invCm2Ev*ecm[ilvl]
+            edgeLvlAng.append(const.ev2Ang/iprLvlEv)
+            iprLvlErg = const.ev2Erg*iprLvlEv
+
+            # scaled energy is relative to the ionization potential for each level
+
+#            scaledE = hnuEv/iprLvlEv
+#            badLong = hnuEv < iprLvlEv
+#            if verbose:
+#                print(' %i nWvl  %i badlong.sum %i'%(ilvl, nWvl,  badLong.sum()))
+#            if badLong.sum() < nWvl:
+            tck = splrep(np.log(pe), np.log(gf),  s=0)
+#            gflog = splev(np.log(scaledE), tck,  der=0,  ext=3)
+#            mygf[ilvl] = np.exp(gflog)
+#            mygf[ilvl][badLong] = 0.
+#                if verbose:
+#                    print(' ilvl %i   mygf.max() %10.2e'%(ilvl, np.max(mygf[ilvl])))
+#            ratg[ilvl] = float(multr[ilvl])/float(mult[0]) # ratio of statistical weights
+
+            for itemp,  atemp in enumerate(self.Temperature):
+                xkt = xgl*const.boltzmann*atemp
+                hnu = xkt + iprLvlErg
+                hnuEv = hnu/const.ev2Erg
+                scaledE = hnuEv/iprLvlEv
+#                badLong = hnuEv < iprLvlEv
+#                tck = splrep(np.log(pe), np.log(gf),  s=0)
+                gflog = splev(np.log(scaledE), tck,  der=0,  ext=3)
+                mygf = np.exp(gflog)
+                mygfInt = (wgl*mygf/hnu).sum()
+                fbRate[ilvl,  itemp] = (K0*ratg[ilvl]*iprLvlErg**2*mygfInt/float(pqn[ilvl]))/np.sqrt(atemp)
+#                    iprLvlErg - hnu)/(const.boltzmann*atemp
+#                    print(' kl %5i %5i  %10.2e'%(ilvl, itemp, atemp))
+                # this test if not really needed
+#                    if ioneq[itemp] > 0.:
+#                    expfun = np.exp((iprLvlErg - hnu)/(const.boltzmann*atemp))
+#                    expfunIsnan = np.isnan(expfun)
+#                    expfunIsinf = np.isinf(expfun)
+#                    if expfunIsnan.sum() > 0:
+#                        print('%i some of expfun are nan %i'%(itemp, expfunIsnan.sum()))
+#                        expfun[expfunIsnan] = 0.
+#                    elif expfunIsinf.sum() > 0:
+#                        print('%i some of expfun are inf %i'%(itemp, expfunIsinf.sum()))
+#                        expfun[expfunIsinf] = 0.
+
+#                if self.Defaults['flux'] == 'energy':
+#                    fbn[ilvl, itemp] = K0*hnu**2*expfun*iprLvlErg**2*ratg[ilvl]*mygf[ilvl]  \
+#                        /(atemp**(1.5)*float(pqn[ilvl]))
+#                elif self.Defaults['flux'] == 'photon':
+#                    fbn[ilvl, itemp] = K0*hnu*expfun*iprLvlErg**2*ratg[ilvl]*mygf[ilvl]  \
+#                        /(atemp**(1.5)*float(pqn[ilvl]))
+
+#                fbnIsNan = np.isnan(fbn[ilvl, itemp])
+#                fbnIsInf = np.isinf(fbn[ilvl, itemp])
+
+#                if fbnIsNan.sum() > 0:
+#                    print('%s  %i some fbn are Nan %i'%(self.IonStr, itemp, fbnIsNan.sum()))
+#                    fbn[ilvl, itemp,  fbnIsNan] = 0.
+#                elif fbnIsInf.sum() > 0:
+#                    print('%s %i some fbn are Inf %i'%(self.IonStr, itemp, fbnIsInf.sum()))
+#                    fbn[ilvl, itemp,  fbnIsInf] = 0.
+#                mask[ilvl,itemp] = 1.e+8/wvl < (self.IprCm - ecm[ilvl])
+#                fbIntensity[ilvl, itemp] = abund*em[itemp]*ioneq[itemp]*fbn[ilvl, itemp]
+#
+        fbTest = fbRate.sum(axis=0)
+        fbTestisNan = np.isnan(fbTest)
+        fbTestisInf = np.isinf(fbTest)
+        if fbTestisNan.sum() > 0:
+            print('fb has some Nans %i'%(fbTestisNan.sum()))
+        if fbTestisInf.sum() > 0:
+            print('fb has some Infs %i'%(fbTestisInf.sum()))
+#        fb = np.ma.array(fbIntensity.sum(axis=0))
+        fb = np.ma.array(fbRate.sum(axis=0))
+#        fb_mask = fb <= 0. or fb == np.inf
+        fb_mask = np.where(fb.data is np.nan,  1,  0)
+        fb.mask = fb_mask
+        fb.fill_value = 0.
+        #
+#        self.FreeBoundRate = {'rate':fbRate, 'temperature':temperature, 'gf':mygf, 'edgeLvlAng':edgeLvlAng,  'fbn':fbn,
+#            'xlabel':xlabel, 'ylabel':ylabel,  'pqn':pqn, 'conf':conf}
+#
+        self.FreeBoundRate = {'rate':fb, 'fbLvl': fbRate,  'temperature':temperature, 'gf':mygf, 'edgeLvlAng':edgeLvlAng,  'fbn':fbn,
+            'xlabel':xlabel, 'ylabel':ylabel,  'pqn':pqn, 'conf':conf}
 
 
 
-    def freeBound(self, wvl, includeAbund = True,  includeIoneq = True,  verbose=False):
+    def freeBound(self, wvl, includeAbund = True, includeIoneq = True, verner=True, verbose=False):
         """
         Calculates the free-bound (radiative recombination) continuum emissivity of an ion.
         Provides emissivity in units of ergs :math:`\mathrm{cm}^{-2}` :math:`\mathrm{s}^{-1}` :math:`\mathrm{str}^{-1}` :math:`\mathrm{\AA}^{-1}` for an individual ion.  If includeAbund is set,
@@ -705,32 +996,33 @@ class continuum(ionTrails):
 
         Notes
         -----
-        - Uses the Gaunt factors of [103]_ for recombination to the ground level
+        - Uses the Gaunt factors of [103]_ for recombination to the excited levels
         - Uses the photoionization cross sections of [3]_ to develop the free-bound cross section
-        - Does not include the elemental abundance or ionization fraction
+        - Include the elemental abundance and ionization fraction by default
         - The specified ion is the target ion
         - uses the corrected version of the K-L bf gaunt factors available in CHIANTI V10
         - revised to calculate the bf cross, fb cross section and the maxwell energy distribution
-        - the Verner cross sections are not included for now
 
         References
         ----------
         .. [3] Verner & Yakovlev, 1995, A&AS, `109, 125
             <http://adsabs.harvard.edu/abs/1995A%26AS..109..125V>`_
         """
+
+        if self.Defaults['wavelength'] != 'angstrom':
+            print(' the continuum can only be calculated for wavelengths in angstroms')
+            return
+
         temperature = self.Temperature
         wvl = np.asarray(wvl, np.float64)
         em = self.Em
 
-        if self.Em.max() == 1.:
-            ylabel = 'erg cm$^{-2}$ s$^{-1}$ sr$^{-1}$ \u212B$^{-1}$ ($\int\,$ N$_e\,$N$_H\,$d${\it l}$)$^{-1}$'
-        else:
-            ylabel = 'erg cm$^{-2}$ s$^{-1}$ sr$^{-1}$ \u212B$^{-1}$'
 
-        if self.Defaults['wavelength'] == 'angstrom':
-            xlabel = 'Wavelength (\u212B)'
-        else:
-            xlabel = r'Wavelength ('+self.Defaults['wavelength'] +')'
+        xlabel = self.Labels['xlabel']
+        ylabel = self.Labels['spectrumYlabel']
+
+        if np.array_equal(self.Em, np.ones_like(self.Em)):
+            ylabel += '($\int\,$ N$_e\,$N$_H\,$d${\it l}$)$^{-1}$'
 
         if includeAbund:
             abund = self.Abundance
@@ -741,7 +1033,7 @@ class continuum(ionTrails):
         else:
             ioneq = np.ones_like(temperature)
         #
-        # the target ion contains that data for fblvl
+        # the target ion contains the data for fblvl
         #
         if hasattr(self,'Fblvl'):
             fblvl = self.Fblvl
@@ -801,16 +1093,17 @@ class continuum(ionTrails):
         nWvl = wvl.size
         nTemp = temperature.size
         #
-        #  the verner cross-section is not included in this version
+        #  the verner cross-section is included in this version
 #        if verner:
 #            self.vernerCross(wvl)
 #            vCross = self.VernerCross
 #            lvl1 = 1
 #        else:
 #            lvl1 = 0
-        lvl1 = 0
-
+#        lvl1 = 0
+        expfun = np.zeros((nlvls,nTemp,nWvl),np.float64)
         mask = np.zeros((nlvls,nTemp,nWvl),np.bool_)
+        expf = np.zeros((nlvls,nTemp,nWvl), np.float64)
         fbn = np.zeros((nlvls,nTemp,nWvl),np.float64)
         fbIntensity = np.zeros((nlvls,nTemp,nWvl),np.float64)
         ratg = np.zeros((nlvls),np.float64)
@@ -820,6 +1113,9 @@ class continuum(ionTrails):
         iprLvlErg = const.ev2Erg*iprLvlEv
         edgeLvlAng = []
 
+        hnu = const.planck*const.light/(1.e-8*wvl)
+        hnuEv = const.ev2Ang/wvl
+
         # constants for free bound
         # K_1 = 1.726 \times 10^{-28} = \frac{2^4 h e^2 }{3 \sqrt 3 m_e c}
         K1 = 2.**4*const.planck*const.q**2/(3.*np.sqrt(3.)*const.emass*const.light)
@@ -830,16 +1126,52 @@ class continuum(ionTrails):
         K3 = (1./(const.planck*const.light))*(1./(np.sqrt(2.*const.emass)))*(1./(np.pi*const.boltzmann)**1.5)
         K0 = 1.e-8*K1*K2*K3
 
-#        for itemp,  atemp  in enumerate(temperature):
-#            if verner:
-##                if verbose:
-##                    print(' calculating verner')
-#                mask[0,itemp] = 1.e+8/wvl < (self.IprCm - ecm[0])
-#                expf[0,itemp] = np.exp((iprLvlErg - 1.e+8*const.planck*const.light/wvl)/(const.boltzmann*atemp))
-#                fbrate[0,itemp] = ioneq[itemp]*em[itemp]*(const.planck*const.light/(1.e-8*wvl))**5 \
-#                    *const.verner*ratg[0]*expf[0,itemp]*vCross/atemp**1.5
+#        V1 = np.sqrt(2.)*const.planck**4*const.light**2/(4*(np.pi*const.emass*const.boltzmann)**(1.5))
         # restrict calculations to temperatures where the ionization equ. is non-zero
-        goodT = [ it for it, anioneq in enumerate(self.IoneqOne) if anioneq > 0.]
+#        goodT = [ it for it, anioneq in enumerate(self.IoneqOne) if anioneq > 0.]
+
+        #  the verner yakovlev photionizaton cross sections
+
+        if verner:
+            lvl1 = 1
+            self.vernerCross(wvl)
+            ilvl = 0
+            iprLvlEv = self.Ipr - const.invCm2Ev*ecm[ilvl]
+            edgeLvlAng.append(const.ev2Ang/iprLvlEv)
+#            for itemp in goodT:
+#                atemp = self.Temperature[itemp]
+##                print(' verner %5i %5i  %10.2e'%(ilvl, itemp, atemp))
+#                mask[0,itemp] = 1.e+8/wvl < (self.IprCm - ecm[0])
+#                fbn[0, itemp] = 1.e-8*V1*atemp**(-1.5) \
+#                    *np.exp((iprLvlErg - 1.e+8*const.planck*const.light/wvl)/(const.boltzmann*atemp)) \
+#                    *(1.e-8/wvl)**5*ratg[0]*self.VernerCross
+##                expfun[0, itemp] = np.exp((iprLvlErg - 1.e+8*const.planck*const.light/wvl)/
+##                    (const.boltzmann*atemp))
+##                fbn[0, itemp] = (const.planck*const.light/(1.e-8*wvl))**5 \
+##                    *const.verner*ratg[0]*expfun[0,itemp]*vCross/atemp**1.5
+#                fbIntensity[0,itemp] = abund*ioneq[itemp]*em[itemp]*fbn[0,  itemp]
+
+            for itemp,  atemp  in enumerate(temperature):
+                mask[0,itemp] = 1.e+8/wvl < (self.IprCm - ecm[0])
+                expf[0,itemp] = np.exp((iprLvlErg - 1.e+8*const.planck*const.light/wvl)/(const.boltzmann*atemp))
+                if self.Defaults['flux'] == 'energy':
+                    fbn[0,itemp] = (const.planck*const.light/(1.e-8*wvl))**5 \
+                        *const.verner*ratg[0]*expf[0,itemp]*self.VernerCross/atemp**1.5
+                elif self.Defaults['flux'] == 'photon':
+                    fbn[0,itemp] = (const.planck*const.light/(1.e-8*wvl))**4 \
+                        *const.verner*ratg[0]*expf[0,itemp]*self.VernerCross/atemp**1.5
+#                fbIntensity[0, itemp] = abund*em[itemp]*ioneq[itemp]*fbn[ilvl, itemp]
+                fbIntensity[0, itemp] = em[itemp]*fbn[ilvl, itemp]
+#                if includeIoneq:
+#                    fbIntensity[0, itemp] *= ioneq[itemp]
+#            if includeAbund:
+#                fbIntensity *= abund
+
+        else:
+            lvl1 = 0
+
+        # the karzas latter gaunt factors are used
+
         for ilvl in range(lvl1,nlvls):
             pqnIdx = pqn[ilvl] - 1
             lIdx = l[ilvl]
@@ -849,8 +1181,6 @@ class continuum(ionTrails):
             iprLvlEv = self.Ipr - const.invCm2Ev*ecm[ilvl]
             edgeLvlAng.append(const.ev2Ang/iprLvlEv)
             iprLvlErg = const.ev2Erg*iprLvlEv
-            hnu = const.planck*const.light/(1.e-8*wvl)
-            hnuEv = const.ev2Ang/wvl
 
             # scaled energy is relative to the ionization potential
 
@@ -860,56 +1190,83 @@ class continuum(ionTrails):
                 print(' %i nWvl  %i badlong.sum %i'%(ilvl, nWvl,  badLong.sum()))
             if badLong.sum() < nWvl:
                 tck = splrep(np.log(pe), np.log(gf),  s=0)
-                gflog = splev(np.log(scaledE), tck,  der=0,  ext=1)
+                gflog = splev(np.log(scaledE), tck,  der=0,  ext=3)
                 mygf[ilvl] = np.exp(gflog)
                 mygf[ilvl][badLong] = 0.
-                if verbose:
-                    print(' ilvl %i   mygf.max() %10.2e'%(ilvl, np.max(mygf[ilvl])))
+#                if verbose:
+#                    print(' ilvl %i   mygf.max() %10.2e'%(ilvl, np.max(mygf[ilvl])))
                 ratg[ilvl] = float(multr[ilvl])/float(mult[0]) # ratio of statistical weights
 
-                for itemp in goodT:
-                    atemp = self.Temperature[itemp]
-                    if ioneq[itemp] > 0.:
-                        phE = hnu
-                        expfun = np.exp((iprLvlErg - phE)/(const.boltzmann*atemp))
-                        expfunIsnan = np.isnan(expfun)
-                        expfunIsinf = np.isinf(expfun)
-                        if expfunIsnan.sum() > 0:
-                            print('%i some of expfun are nan %i'%(itemp, expfunIsnan.sum()))
-                            expfun[expfunIsnan] = 0.
-                        elif expfunIsinf.sum() > 0:
-                            print('%i some of expfun are inf %i'%(itemp, expfunIsinf.sum()))
-                            expfun[expfunIsinf] = 0.
-                        fbn[ilvl, itemp] = K0*phE**2*expfun*iprLvlErg**2*ratg[ilvl]*mygf[ilvl]/(atemp**(1.5)*float(pqn[ilvl]))
-                        fbnIsNan = np.isnan(fbn[ilvl, itemp])
-                        fbnIsInf = np.isinf(fbn[ilvl, itemp])
+            for itemp,  atemp  in enumerate(temperature):
+#                    atemp = self.Temperature[itemp]
+#                    print(' kl %5i %5i  %10.2e'%(ilvl, itemp, atemp))
+                # this test if not really needed
+#                    if ioneq[itemp] > 0.:
+                expfun = np.exp((iprLvlErg - hnu)/(const.boltzmann*atemp))
+                expfunIsnan = np.isnan(expfun)
+                expfunIsinf = np.isinf(expfun)
+                if expfunIsnan.sum() > 0:
+                    print('%i some of expfun are nan %i'%(itemp, expfunIsnan.sum()))
+                    expfun[expfunIsnan] = 0.
+                elif expfunIsinf.sum() > 0:
+                    print('%i some of expfun are inf %i'%(itemp, expfunIsinf.sum()))
+                    expfun[expfunIsinf] = 0.
 
-                        if fbnIsNan.sum() > 0:
-                            print('%s  %i some fbn are Nan %i'%(self.IonStr, itemp, fbnIsNan.sum()))
-                            fbn[ilvl, itemp,  fbnIsNan] = 0.
-                        elif fbnIsInf.sum() > 0:
-                            print('%s %i some fbn are Inf %i'%(self.IonStr, itemp, fbnIsInf.sum()))
-                            fbn[ilvl, itemp,  fbnIsInf] = 0.
-                        mask[ilvl,itemp] = 1.e+8/wvl < (self.IprCm - ecm[ilvl])
-                        fbIntensity[ilvl, itemp] = abund*em[itemp]*ioneq[itemp]*fbn[ilvl, itemp]
+                if self.Defaults['flux'] == 'energy':
+                    fbn[ilvl, itemp] = K0*hnu**2*expfun*iprLvlErg**2*ratg[ilvl]*mygf[ilvl]  \
+                        /(atemp**(1.5)*float(pqn[ilvl]))
+                elif self.Defaults['flux'] == 'photon':
+                    fbn[ilvl, itemp] = K0*hnu*expfun*iprLvlErg**2*ratg[ilvl]*mygf[ilvl]  \
+                        /(atemp**(1.5)*float(pqn[ilvl]))
 
+                fbnIsNan = np.isnan(fbn[ilvl, itemp])
+                fbnIsInf = np.isinf(fbn[ilvl, itemp])
+
+                if fbnIsNan.sum() > 0:
+                    print('%s  %i some fbn are Nan %i'%(self.IonStr, itemp, fbnIsNan.sum()))
+                    fbn[ilvl, itemp,  fbnIsNan] = 0.
+                elif fbnIsInf.sum() > 0:
+                    print('%s %i some fbn are Inf %i'%(self.IonStr, itemp, fbnIsInf.sum()))
+                    fbn[ilvl, itemp,  fbnIsInf] = 0.
+                mask[ilvl,itemp] = 1.e+8/wvl < (self.IprCm - ecm[ilvl])
+#                fbIntensity[ilvl, itemp] = abund*em[itemp]*ioneq[itemp]*fbn[ilvl, itemp]
+                fbIntensity[ilvl, itemp] = em[itemp]*fbn[ilvl, itemp]
+#                if includeIoneq:
+#                    fbIntensity[ilvl, itemp] *= ioneq[itemp]
+#            if includeAbund:
+#                fbIntensity *= abund
         fbTest = fbIntensity.sum(axis=0)
+        fbTest *= self.Abundance
+
+        for itemp, one in enumerate(self.IoneqOne):
+            fbTest[itemp] *= one
+
         fbTestisNan = np.isnan(fbTest)
         fbTestisInf = np.isinf(fbTest)
         if fbTestisNan.sum() > 0:
             print('fb has some Nans %i'%(fbTestisNan.sum()))
         if fbTestisInf.sum() > 0:
             print('fb has some Infs %i'%(fbTestisInf.sum()))
-        fb = np.ma.array(fbIntensity.sum(axis=0))
+        fbmask = np.logical_and(fbTestisNan,  fbTestisInf)
+
+        fb = np.ma.masked_array(fbTest,   mask=fbmask)
+#        fb = np.ma.array(fbIntensity.shape)
+#        fb = np.ma.array(fbIntensity.sum(axis=0))
+#        fb *= abund
+#        for itemp in enumerate(temperature):
+#            fb[itemp] *= ioneq[itemp]
 #        fb_mask = fb <= 0. or fb == np.inf
-        fb_mask = np.where(fb.data is np.nan,  1,  0)
-        fb.mask = fb_mask
+#        fb.data = fbIntensity.sum(axis=0)
+#        fb_mask = np.where(fb.data is np.nan,  1,  0)
+#        fb.mask = fb_mask
         fb.fill_value = 0.
         #
         self.FreeBound = {'intensity':fb.squeeze(), 'temperature':temperature,'wvl':wvl, 'em':em, \
-            'abund':abund, 'ioneq':ioneq, 'gf':mygf, 'edgeLvlAng':edgeLvlAng,  'fbn':fbn,
+            'abund':abund, 'ioneq':ioneq, 'gf':mygf, 'edgeLvlAng':edgeLvlAng,  'fbn':fbn.squeeze(),
             'xlabel':xlabel, 'ylabel':ylabel}
 
+
+
     def vernerCross(self, wvl):
         """
         Calculates the photoionization cross-section using data from [102]_ for
@@ -926,9 +1283,14 @@ class continuum(ionTrails):
         :math:`\sigma_0,E_0,y_w,y_a,P` are fitting paramters. These can be read in using
         `ChiantiPy.tools.io.vernerRead`.
 
+        Parameters:
+
+        wvl : `list`, `ndarray`
+            wavelength in Angstroms
+
         """
         # read verner data
-        verner_info = io.vernerRead()
+        verner_info = chdata.Verner_info
         eth = verner_info['eth'][self.Z,self.Stage-1]   #*const.ev2Erg
         yw = verner_info['yw'][self.Z,self.Stage-1]
         ya = verner_info['ya'][self.Z,self.Stage-1]
@@ -940,7 +1302,7 @@ class continuum(ionTrails):
         q = 5.5 + verner_info['l'][self.Z,self.Stage-1] - 0.5*p
 
         # scaled photon energy
-        en = const.ev2Ang/wvl
+        en = const.ev2Ang/np.asarray(wvl)
         y = en/e0
         # fitting function
         F = ((y - 1.)**2 + yw**2)*(y**(-q))*(1. + np.sqrt(y/ya))**(-p)
